//#define Debug

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using RFID;
using RFID.Logging;

namespace GuiTestApi
{
    public partial class ID_MAIN_FORM : Form
    {
		private RFIDModule rfidmodule;
        private delegate void UpdateGuiDelegate(byte[] data, ReceivedMessage message);
        private UpdateGuiDelegate updateGuiDelegate;
        private delegate void SendCallback();
        private RFIDModule.RFModes modes;
        private delegate void StartProgressbarDelegate();
        private bool connecting = false;
        private bool connected = false;
        private byte[] previousData = null;

        public ID_MAIN_FORM()
        {
            InitializeComponent();

            ID_TEXTBOX_MESSAGE_WND.ContextMenu = ID_CONTEXT_MENU_MESSAGE_WND;
            ID_CHECKBOX_FDX.CheckStateChanged += new EventHandler(OnCheckboxStateChanged);
            ID_CHECKBOX_H4002.CheckStateChanged += new EventHandler(OnCheckboxStateChanged);
            ID_CHECKBOX_HDX.CheckStateChanged += new EventHandler(OnCheckboxStateChanged);

            //set loglevel to 0 so all logging can take place
            Log.LogLevel = Log.LogLevels.Level_2;

            init();
        }
		
		public void init()
        {
            rfidmodule = new RFIDModule();

            rfidmodule.SetBTState(true);

            ID_LABEL_VERSION.Text = "RFID reader application " + rfidmodule.APIVersion + " Handheld Group";

            rfidmodule.ModuleDataReceivedEvent += new RFIDModule.ModuleDataReceived(rfidmodule_moduleDataReceivedEvent);
            //rfidmodule.ModuleDataSendEvent += new RFIDModule.ModuleDataSend(rfidmodule_moduleDataSendEvent);
            rfidmodule.ModuleOpenedEvent += new RFIDModule.ModuleOpened(rfidmodule_moduleOpenedEvent);
            rfidmodule.ModulePowerStateChangedEvent += new RFIDModule.ModulePowerStateChanged(rfidmodule_modulePowerStateChangedEvent);
            rfidmodule.ModuleConnectionStateChangedEvent += new RFIDModule.ModuleConnectionStateChanged(rfidmodule_ModuleConnectionStateChangedEvent);
            rfidmodule.ModuleReceiveThreadExceptionOccurredEvent += new RFIDModule.ModuleReceiveThreadExceptionOccurred(rfidmodule_ModuleReceiveThreadExceptionOccurredEvent);
            //prevent the device from going to sleep mode
            rfidmodule.PreventDeviceSleep();

            updateGuiDelegate = new UpdateGuiDelegate(UpdateGui);
        }

        void rfidmodule_ModuleReceiveThreadExceptionOccurredEvent(ReceiveThreadExceptionsEventargs e)
        {
            Log.Write("An exception occurred in the receive thread and the receivethread is closed", Log.LogType.Exception);
            rfidmodule.Close();

            //MessageBox.Show(String.Format("An exception occurred while receiving data, details: {0}. The connection to the RFID module is closed", e.ErrorMessage),"Exception", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
        }

        /// <summary>
        /// This event is triggered when no tag is detected for a period of time
        /// </summary>
        void rfidmodule_modulePowerStateChangedEvent()
        {
            rfidmodule.Close();

            if (this.InvokeRequired)
            {
                this.BeginInvoke(new EventHandler(delegate
                {
                    ID_RADIO_BUTTON_CONNECTED.Checked = false;
                    ID_PANEL_CONFIG.Enabled = false;
                    ID_PANEL_RECEIVED.Enabled = false;
                    ID_PANEL_STATE.Enabled = false;
                }));
            }

            if (MessageBox.Show("RFID reader going into low power mode, click no to prevent this", "Alert", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1) == DialogResult.No)
            {
                rfidmodule.BeginOpen();
                rfidmodule.StartReadTimeOut();
            }
        }

        void rfidmodule_moduleDataSendEvent(object sender, DataSendEventArgs e)
        {
            if (!e.SendSuccesfull)
            {
                if (rfidmodule.Connected)
                {
                    if (this.InvokeRequired)
                    {
                        this.BeginInvoke(new EventHandler(delegate
                        {
                            ID_RADIO_BUTTON_CONNECTED.Checked = false;
                        }));
                    }
                }
                else
                {
                    MessageBox.Show("connection is up but writing data generated an error");
                }               
            }
        }

        void rfidmodule_moduleOpenedEvent(object sender, ModuleOpenEventArgs e)
        {
            if (e.Connected)
            {
                Log.Write("A connection is made successful, openedEvent triggered", Log.LogType.Debug);
                if (this.InvokeRequired)
                {
                    this.BeginInvoke(new EventHandler(delegate
                    {
                        rfidmodule.GetRFMode();

                        ID_PANEL_CONFIG.Enabled = true;
                        ID_PANEL_STATE.Enabled = true;
                        ID_PANEL_RECEIVED.Enabled = true;
                        ID_BUTTON_AUTO_DETECT.Enabled = true;
                        ID_BUTTON_SINGLE_READ.Enabled = true;
                        ID_RADIO_BUTTON_CONNECTED.Checked = true;
                        ID_BUTTON_CONNECT.Enabled = false;
                        ID_BUTTON_DISCONNECT.Enabled = true;
                        //reset connecting flag since connecting has finished
                        connecting = false;

                        RFID.Logging.Log.Write("Everything is enabled", Log.LogType.Debug);

                    }));
                }
                //set timeout value in minutes
                rfidmodule.RFIDModuleTimeout = 10;

                rfidmodule.StartReadTimeOut();

                rfidmodule.BeginReceiving();
                //rfidmodule.SetTimeout(10000);
                connected = true;
            }
            else
            {
                DialogResult result = MessageBox.Show("While connecting to the device something went wrong, try again? Reason: " + e.ConnectionError, "Connection Error", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1);

                if (result == DialogResult.Yes)
                {
                    try
                    {
                        //close previous connection session
                        rfidmodule.Close();
                        //start new connection session
                        rfidmodule.BeginOpen();
                        //reset connecting flag
                        connecting = false;
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("An exception occurred while calling BeginOpen() from form1.cs, details: " + ex.Message);
                    }
                }
                else if (result == DialogResult.No)
                {
                    rfidmodule.Exit();

                    Application.Exit();
                }
            }    
        }

        /// <summary>
        /// Receives data received events
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rfidmodule_moduleDataReceivedEvent(object sender, DataEventArgs e)
        {
            byte[] buffer = e.RawData;

            ReceivedMessage receivedMessage = e.Message;

            object[] MethodArgs = new object[] { buffer, receivedMessage };
            if (this.InvokeRequired)
            {
                ID_TEXTBOX_MESSAGE_WND.BeginInvoke(updateGuiDelegate, MethodArgs);
            }
        }

        /// <summary>
        /// receives connection state changes. An connection state event is triggered when
        /// the power state changes from Suspend to On State
        /// </summary>
        /// <param name="e"></param>
        void rfidmodule_ModuleConnectionStateChangedEvent(ConnectionStateChangedArgs e)
        {
            if (e.ConnectionState == RFIDModule.ModuleConnectionState.Disconnected)
            {
                //check if previous state was connected (note cannot check 
                if (connected)
                {
                    if (this.InvokeRequired)
                    {
                        ID_RADIO_BUTTON_CONNECTED.BeginInvoke(new EventHandler(delegate
                        {
                            ID_RADIO_BUTTON_CONNECTED.Checked = false;
                            ID_PANEL_CONFIG.Enabled = false;
                            ID_PANEL_RECEIVED.Enabled = false;
                            ID_PANEL_STATE.Enabled = false;
                        }));
                    }

                    DialogResult result = MessageBox.Show("The connection to the device is lost. Would you try to reconnect? ", "Connection lost", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);

                    if (result == DialogResult.Yes)
                    {
                        Log.Write("Reconnect", Log.LogType.Debug);
                        rfidmodule.Close();
                        rfidmodule.BeginOpen();
                    }
                }
            }
        }

        /// <summary>
        /// This method updates the gui components when a new message is received.
        /// </summary>
        /// <param name="data"></param>
        /// <param name="message"></param>
		private void UpdateGui(byte[] data, ReceivedMessage message)
        {
            //if (!Helper.ConvertByteToHexString(data).Equals(Helper.ConvertByteToHexString(previousData)))
            //{
                if (message.TypeOfMessage == ReceivedMessage.MessageType.Tag)
                {

                    ID_TEXTBOX_COUNTRY_CODE.Text = message.ReadTag.CountryCode;
                    ID_TEXTBOX_TAG_ID.Text = message.ReadTag.TagID;
                }

                ID_TEXTBOX_MESSAGE_WND.Text += RFID.Helper.ConvertByteToHexString(data) + "\r\n";

                if (ID_TEXTBOX_MESSAGE_WND.TextLength >= 32767)
                    ID_TEXTBOX_MESSAGE_WND.Text = "";
            //}

            //previousData = data;
        }
        
        private void ID_BUTTON_EXIT_Click(object sender, EventArgs e)
        {
            Log.Write("Exit button clicked", Log.LogType.Debug);
            rfidmodule.Exit();

            Application.Exit();
        }


        private void ID_MENUITEM_CLEAR_Click(object sender, EventArgs e)
        {
            ID_TEXTBOX_MESSAGE_WND.Text = "";
        }

        private void ID_BUTTON_SINGLE_READ_Click(object sender, EventArgs e)
        {
            if(!ID_BUTTON_SINGLE_READ.BackColor.Equals(Color.FromArgb(0,128,0)))
            {
                ID_BUTTON_SINGLE_READ.BackColor = Color.Green;
                ID_BUTTON_AUTO_DETECT.BackColor = Color.FromArgb(192, 224, 255);
                rfidmodule.SetOperationMode(RFIDModule.OperationModes.SlaveRequest);
            }
            rfidmodule.GetTimeout();
            rfidmodule.GetSingleID();
        }

        private void ID_BUTTON_AUTO_DETECT_Click(object sender, EventArgs e)
        {
            if (ID_BUTTON_SINGLE_READ.BackColor.Equals(Color.FromArgb(0,128,0)))
            {
                ID_BUTTON_SINGLE_READ.BackColor = Color.FromArgb(192, 224, 255);
                ID_BUTTON_AUTO_DETECT.BackColor = Color.Green;
                rfidmodule.SetOperationMode(RFIDModule.OperationModes.Auto);
            }
        }

        private void ID_TEXTBOX_MESSAGE_WND_TextChanged(object sender, EventArgs e)
        {
            ID_TEXTBOX_MESSAGE_WND.Select(ID_TEXTBOX_MESSAGE_WND.Text.Length, 0);
            ID_TEXTBOX_MESSAGE_WND.ScrollToCaret();
        }

        private void OnCheckboxStateChanged(object sender, EventArgs e)
        {
            bool all = (ID_CHECKBOX_FDX.Checked && ID_CHECKBOX_HDX.Checked && ID_CHECKBOX_H4002.Checked);
            bool hdx = ID_CHECKBOX_HDX.Checked;
            bool fdx = ID_CHECKBOX_FDX.Checked;
            bool h4002 = ID_CHECKBOX_H4002.Checked;
           
            //enable the save button
            ID_BUTTON_SAVE.Enabled = true;
             
            //if fdx is checked
            if (fdx && !hdx && !h4002)
            {
                modes = RFIDModule.RFModes.FDX;
            }
            //if hdx and fdx are checked
            else if (hdx && fdx && !h4002)
            {
                modes = RFIDModule.RFModes.FDX | RFIDModule.RFModes.HDX;
            }
            //if hdx is checked
            else if (hdx && !fdx && !h4002)
            {
                modes = RFIDModule.RFModes.HDX;
            }
            //if hdx and h4002 are checked
            else if (hdx && h4002 && !fdx)
            {
                modes = RFIDModule.RFModes.HDX | RFIDModule.RFModes.H4002;
            }
            //if fdx and h4002 are checked
            else if (fdx && h4002 && !hdx)
            {
                modes = RFIDModule.RFModes.FDX | RFIDModule.RFModes.H4002;
            }
            //if h4002 is checked
            else if (h4002 && !hdx && !fdx)
            {
                modes = RFIDModule.RFModes.H4002;
            }
            //if none then go to default hdx and fdx active
            else if (!hdx && !fdx && !h4002)
            {
                modes = RFIDModule.RFModes.FDX | RFIDModule.RFModes.HDX;
            }
            //if all are checked
            else
            {
                modes = RFIDModule.RFModes.HDX | RFIDModule.RFModes.H4002 | RFIDModule.RFModes.FDX;               
            }
        }

        private void ID_BUTTON_SAVE_Click(object sender, EventArgs e)
        {
            //write setting
            rfidmodule.SetRFMode(modes);
            //save setting
            rfidmodule.SaveConfig();
            //read setting
            rfidmodule.GetRFMode();
            //disable the save button
            ID_BUTTON_SAVE.Enabled = false;
        }

        private void ID_BUTTON_GET_CONFIG_Click(object sender, EventArgs e)
        {
            Settings settingsForm = new Settings(rfidmodule);
            settingsForm.ShowDialog();
        }

        private void ID_BUTTON_RESET_SETTINGS_Click(object sender, EventArgs e)
        {
            rfidmodule.Reset();
            ID_BUTTON_SINGLE_READ.BackColor = Color.FromArgb(192, 224, 255);
            ID_BUTTON_AUTO_DETECT.BackColor = Color.Green;
        }

        private void ID_BUTTON_CONNECT_Click(object sender, EventArgs e)
        {
            try
            {
                if (!connecting)
                {

                        rfidmodule.BeginOpen();
                        connecting = true;

                }
                else
                {
                    MessageBox.Show("Already trying to connect", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                }
            }
            catch (Exception ex)
            {
                Log.Write("Cannot connect to rfid device, details: " + ex.Message, Log.LogType.Exception);
            }
        }

        private void ID_BUTTON_DISCONNECT_Click(object sender, EventArgs e)
        {
            try
            {
                rfidmodule.Close();

                ID_BUTTON_DISCONNECT.Enabled = false;
                ID_BUTTON_CONNECT.Enabled = true;
                connected = false;
            }
            catch (Exception ex)
            {
                Log.Write("Cannot disconnect rfid device, details: " + ex.Message, Log.LogType.Exception);
            }
        }
    }
}